@@ -41,16 +41,28 @@ warnings.filterwarnings(
 )
 
 warnings.filterwarnings("ignore", message="Gdk.Cursor.new is deprecated")
+warnings.filterwarnings(
+    "ignore",
+    message=(
+        " `np.bool` is a deprecated alias for the builtin `bool`. "
+        "To silence this warning, use `bool` by itself. Doing this will not modify any "
+        "behavior and is safe. If you specifically wanted the numpy scalar type, "
+        "use `np.bool_` here."
+    ),
+)
+warnings.filterwarnings(
+    "ignore",
+    message=" WARN: Box bound precision lowered by casting to float32",
+)
 try:
     from plangym.atari import AtariEnvironment  # noqa: E402
-    from plangym.classic_control import ClassicControl  # noqa: E402
-    from plangym.core import BaseEnvironment  # noqa: E402
-    from plangym.dm_control import DMControlEnv, ParallelDMControl  # noqa: E402
-    from plangym.minimal import MinimalPacman, MinimalPong  # noqa: E402
-    from plangym.montezuma import Montezuma  # noqa: E402
-    from plangym.parallel import ParallelEnvironment  # noqa: E402
-    from plangym.retro import ParallelRetro, RetroEnvironment  # noqa: E402
-
 except ImportError as e:
     warnings.warn("Failed to import module in plangym.__init__.py: %s" % str(e))
+from plangym.classic_control import ClassicControl  # noqa: E402
+from plangym.core import BaseEnvironment  # noqa: E402
+from plangym.dm_control import DMControlEnv  # noqa: E402
+from plangym.minimal import MinimalPacman, MinimalPong  # noqa: E402
+from plangym.montezuma import Montezuma  # noqa: E402
+from plangym.parallel import ParallelEnvironment  # noqa: E402
+from plangym.retro import RetroEnvironment  # noqa: E402
 from plangym.version import __version__  # noqa: E402
