@@ -1,14 +1,14 @@
 import pytest
 
 from plangym.classic_control import ClassicControl
-from tests.api_tests import batch_size, TestGymEnvironment
+from tests.api_tests import batch_size, display, TestBaseEnvironment, TestGymEnvironment
 
 
 environments = [
     "MountainCar-v0",
     "Acrobot-v1",
     "MountainCarContinuous-v0",
-    "Pendulum-v0",
+    "Pendulum-v1",
     "CartPole-v0",
 ]
 
